const client = require('prom-client')
const pkg = require('../package.json')

const yves = require('yves')
// const debugMetrics = yves.debugger(`${pkg.name.replace(/-/g,':')}:metrics`)

const debugReport = yves.debugger(`${pkg.name.replace(/-/g,':')}:report`)
const debugError = yves.debugger(`${pkg.name.replace(/-/g,':')}:error`)

const sortobject = require('deep-sort-object')
const util = require('./util')


const metric = {
  web: {
    requests: {
      duration: new client.Summary({ name: 'node_yonderbox_graphql_web_request_duration_milliseconds', help: 'web request duration in milliseconds', labelNames: ['path', 'method', 'status'] }),
      buckets: new client.Histogram({ name: 'node_yonderbox_graphql_web_request_buckets_milliseconds', help: 'web request duration buckets in milliseconds. Bucket size set to 500 and 2000 ms to enable apdex calculations with a T of 300ms', labelNames: ['path', 'method', 'status'], buckets: [ 500, 2000 ] })
    }
  },
}

function report_frontside(method, url, statusCode, start) {
  const duration = util.duration(start)
  if (typeof url === 'string') {
    metric.web.requests.duration.labels(url, method || 'GET', statusCode).observe(duration)
    metric.web.requests.buckets.labels(url, method || 'GET', statusCode).observe(duration)
  }
}

function middleware (req, res, done) {
  const start = util.duration()

  res.on('finish', function () {
    if (res.report) debugReport('%y',sortobject(res.report))
    debugReport('method %y\npayload %y\nheaders %y\nduration %y ms\nstatus: %y',req.method,req.method == 'GET' ? req.query : req.body, req.headers,util.duration(start),res.statusCode)
    if (res.statusCode != 200) {
      debugError('method %y\npayload %y\nheaders %y\nduration %y ms\nstatus: %y\nmessage: %y',req.method,req.method == 'GET' ? req.query : req.body, req.headers,util.duration(start),res.statusCode, res.statusMessage)
    }
    report_frontside(req.method || 'GET', req.path, res.statusCode, start)
  })

  return done()
}

function instrument(server, options) {
  server.use(middleware)
  server.get((options && options.url) || '/metrics', (req, res) => {
    res.header('content-type', 'text/plain; charset=utf-8')
    return res.send(client.register.metrics())
  })
}


module.exports = {
  instrument,
}
